@@ -213,7 +213,7 @@ bool GetDisks(std::list<DiskInfo>& list)
 
     if (GetVolumeInformationW(info.name.c_str(),buffer2,MAX_PATH+1,NULL,&compLength,&flags,NULL,0)!=0) {
       info.dir=buffer2;
-      
+
       if (info.dir==L"OS") {
 	      info.dir=info.name;
       }
@@ -296,7 +296,7 @@ class DirEntry
 {
 public:
   Lum::Base::Path                  path;
-  unsigned long                    size;
+  double                           size;
   std::map<std::wstring,DirEntry*> subDirs;
 
 public:
@@ -342,14 +342,16 @@ private:
 
 private:
   DirEntry              *top;
+  double                size;
 
   std::list<Rect>       labels;
 
   Lum::Model::StringRef label;
 
 public:
-  Map(DirEntry *top)
+  Map(DirEntry *top, double size)
    : top(top),
+     size(size),
      label(new Lum::Model::String(L""))
   {
     // no code
@@ -381,14 +383,14 @@ public:
                  int x, int y, size_t width, size_t height,
                  int depth)
   {
-    size_t full=entry->size;
+    size_t full=lround(entry->size);
     size_t area=width*height;
     int    offset;
 
     // TODO: Instead dividing hardcode by 1024 to avoid overlow,
     // Calculate factor based on the full value!
 
-    if (area==0 || full==0 || full/1024==0) {
+    if (area==0 || full==0) {
       return;
     }
 
@@ -411,7 +413,7 @@ public:
 
       rect.label=iter->second->path.GetPath();
 
-      size_t subArea=((area/1024)*iter->second->size)/(full/1024);
+      size_t subArea=lround((area*iter->second->size)/full);
 
       if (subArea==0) {
         continue;
@@ -485,6 +487,7 @@ public:
     }
 
     Lum::OS::DrawInfo *draw=GetDrawInfo();
+    size_t            fillFactor;
 
     draw->PushForeground(Lum::OS::Display::whiteColor);
     draw->FillRectangle(this->x,this->y,width,height);
@@ -494,8 +497,12 @@ public:
     draw->DrawRectangle(this->x,this->y,width,height);
     draw->PopForeground();
 
+    fillFactor=lround((100*top->size)/size);
+
+    std::cout << fillFactor << std::endl;
+
     labels.clear();
-    DrawEntry(draw,top,this->x+1,this->y+1,width-2,height-2,1);
+    DrawEntry(draw,top,this->x+1,this->y+1,((width-2)*fillFactor)/100,height-2,1);
   }
 
   void ShowLabel(int x, int y)
@@ -540,12 +547,14 @@ class MapWindow : public Lum::Dialog
 {
 private:
   DirEntry *top;
+  double   size;
 
 public:
-  MapWindow(DirEntry *top)
-   : top(top)
+  MapWindow(DirEntry *top, double size)
+   : top(top),
+     size(size)
   {
-    AttachModel(GetClosedAction());
+    Observe(GetClosedAction());
   }
 
   void PreInit()
@@ -560,7 +569,7 @@ public:
     vert->SetWidth(Lum::Base::Size::workHRel,80);
     vert->SetHeight(Lum::Base::Size::workVRel,80);
 
-    map=new Map(top);
+    map=new Map(top,size);
     map->SetFlex(true,true);
 
     vert->Add(map);
@@ -598,14 +607,21 @@ private:
 
 public:
   DirEntry        *top;
+  double          size;
 
 public:
   ScanTask()
-   : top(NULL)
+   : top(NULL),
+     size(0)
   {
     // no code
   }
 
+  void SetSize(double size)
+  {
+    this->size=size;
+  }
+
   void SetDir(const Lum::Base::Path& dir)
   {
     this->dir=dir;
@@ -742,26 +758,17 @@ public:
     scanTask.SetParent(this);
     scanTask.SetCaption(L"Scanning directory...");
 
-    AttachModel(about);
-    AttachModel(showDetail);
-    AttachModel(showMap);
-    AttachModel(refreshTime);
-    AttachModel(timer);
-    AttachModel(GetOpenedAction());
+    Observe(about);
+    Observe(showDetail);
+    Observe(showMap);
+    Observe(refreshTime);
+    Observe(timer);
+    Observe(GetOpenedAction());
 
-    AttachModel(scanTask.GetFinishedAction());
+    Observe(scanTask.GetFinishedAction());
   }
 
-  ~MainWindow()
-  {
-    UnattachModel(GetOpenedAction());
-    UnattachModel(timer);
-    UnattachModel(refreshTime);
-    UnattachModel(showDetail);
-    UnattachModel(about);
-  }
-
-  virtual void PreInit()
+  void PreInit()
   {
     Lum::Label            *label;
     Lum::Panel            *panel,*panel2;
@@ -919,7 +926,7 @@ public:
   {
     std::list<DiskInfo>::const_iterator iter;
 
-    if (selection->HasSelection()){
+    if (selection->HasSelection()) {
       Lum::Model::ListTable::Entry *entry=tableModel->GetEntry(selection->GetLine());
 
       iter=list.begin();
@@ -977,6 +984,7 @@ public:
 
     path.SetNativeDir(iter->dir);
 
+    scanTask.SetSize(iter->blocks*iter->unit);
     scanTask.SetDir(path);
     scanTask.Start();
   }
@@ -989,7 +997,7 @@ public:
     top=scanTask.top;
     scanTask.top=NULL;
 
-    window=new MapWindow(top);
+    window=new MapWindow(top,top->size/*scanTask.size*/);
     window->SetParent(this);
 
     if (window->Open()) {
